"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isCalledAsTagged = exports.splitConditionsOtherwise = exports.hasDuplicatedRowExcept = exports.getOutputFromArray = void 0;
const collection_1 = require("./collection");
const deep_equal_1 = __importDefault(require("deep-equal"));
const getOutputFromArray = (headers, row, symbol) => {
    let result = symbol.undefined;
    for (const [index, cellValue] of row.entries()) {
        if (cellValue === symbol.any || cellValue === symbol.otherwise)
            continue;
        if (headers[index] === symbol.out) {
            result = cellValue;
            continue;
        }
        if (!(0, deep_equal_1.default)(headers[index], cellValue)) {
            // Not matched this row.
            return symbol.undefined;
        }
    }
    return result;
};
exports.getOutputFromArray = getOutputFromArray;
const hasDuplicatedRowExcept = (rows, exceptIndex) => {
    const conditions = rows.map((row) => row.filter((_, index) => index !== exceptIndex));
    return (0, collection_1.hasDuplicated)(conditions);
};
exports.hasDuplicatedRowExcept = hasDuplicatedRowExcept;
const splitConditionsOtherwise = (rows, symbols) => {
    const normal = rows.filter((row) => !row.includes(symbols.otherwise));
    const otherwise = rows.filter((row) => row.includes(symbols.otherwise));
    return [normal, otherwise];
};
exports.splitConditionsOtherwise = splitConditionsOtherwise;
const isCalledAsTagged = (strings, vars) => {
    if (!strings)
        return false;
    if (!strings.raw)
        return false;
    return [
        strings.length > 0,
        strings.raw.length === strings.length,
        Object.isFrozen(strings),
        vars.length + 1 === strings.length,
    ].every((condition) => condition === true);
};
exports.isCalledAsTagged = isCalledAsTagged;
