"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThinRichTable = void 0;
const collection_1 = require("./collection");
const exception_1 = require("./exception");
const hepler_1 = require("./hepler");
class ThinRichTable {
    out;
    cell;
    internals;
    constructor() {
        this.out = Symbol("out");
        this.cell = {
            any: Symbol("any"),
            otherwise: Symbol("otherwise"),
        };
        this.internals = {
            undefined: Symbol("undefined"),
        };
    }
    #getOutputFromArray(headers, row) {
        const symbols = { ...this.cell, ...this.internals, out: this.out };
        return (0, hepler_1.getOutputFromArray)(headers, row, symbols);
    }
    #hasDuplicatedConditions(rows, outputIndex) {
        return (0, hepler_1.hasDuplicatedRowExcept)(rows, outputIndex);
    }
    eval(strings, ...vars) {
        if (!(0, hepler_1.isCalledAsTagged)(strings, vars)) {
            throw exception_1.TRExceptions.notCalledAsTaggedError;
        }
        // `this.out` needs to be defined on the end of table header.
        const columnIndex = vars.indexOf(this.out);
        if (columnIndex < 0) {
            throw exception_1.TRExceptions.notDefinedOutputError;
        }
        const outputColumns = vars.filter((variable) => variable === this.out);
        if (outputColumns.length !== 1) {
            throw exception_1.TRExceptions.duplicatedOutputError;
        }
        const columnCount = columnIndex + 1;
        if (vars.length % columnCount !== 0) {
            throw exception_1.TRExceptions.notMatchedColumnCountError;
        }
        const splitted = (0, collection_1.splitArrayAsNLength)(vars, columnCount);
        const headers = splitted[0];
        const rows = splitted.splice(1);
        if (this.#hasDuplicatedConditions(rows, columnIndex)) {
            throw exception_1.TRExceptions.duplicatedConditionError;
        }
        const matched = [];
        const [normalRows, otherwiseRows] = (0, hepler_1.splitConditionsOtherwise)(rows, {
            ...this.cell,
        });
        for (const rows of [normalRows, otherwiseRows]) {
            for (const row of rows) {
                const output = this.#getOutputFromArray(headers, row);
                if (output === this.internals.undefined)
                    continue;
                matched.push({ row, output });
            }
            if (matched.length === 1) {
                return matched[0].output;
            }
            else if (matched.length > 1) {
                throw exception_1.TRExceptions.duplicatedConditionError;
            }
        }
        throw exception_1.TRExceptions.notCoveredConditionError;
    }
}
exports.ThinRichTable = ThinRichTable;
